package com.sillycycle.bagleyd.panex.view;

/*
 * @(#)TileDrawable.java
 *
 * Copyright 2015  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

/*import java.awt.Canvas;
import java.awt.Color;
import java.awt.Font;
import java.awt.Point;
import java.awt.Graphics;
import java.awt.Image;*/

//import android.graphics.Canvas;
import android.graphics.Canvas;
import android.graphics.ColorFilter;
import android.graphics.Paint.Style;
import android.graphics.Path;
import android.graphics.PixelFormat;
import android.graphics.Point;
import android.graphics.drawable.Drawable;
import android.graphics.Paint;
import android.graphics.Rect;

import com.sillycycle.bagleyd.util.Colour;
import com.sillycycle.bagleyd.panex.model.PanexFormat;

// Saw the light looking at
// http://stackoverflow.com/questions/17950258/how-to-programmatically-create-or-alter-a-drawable-made-of-lines-of-different-co

/**
 * The <code>PanexDraw</code> class does all the basic graphics for
 * <code>PanexApplet</code> class.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/panex.html
 */

public class TileDrawable extends Drawable {
	private Paint mPaint;
	private Rect mRect;
	private int mNumber;
	private int mSize;
	private int mColor;
	private boolean mPressed = false;
	private Point[] trapazoidUnit = new Point[5];
	private Point delta = new Point(8, 2);
	int background = Colour.gray25;
	int tileColor = Colour.gray75;
	int[] pyramidColor = new int[PanexFormat.MAX_COLUMNS - 1];
	private boolean mono = false, reverse = false;

	public TileDrawable(Rect rect, int number, int color, int size) {
		mPaint = new Paint();
		mRect = rect;
		mNumber = number;
		mColor = color;
		mSize = size;
		trapazoidUnit[0] = new Point(0, 0);
		trapazoidUnit[1] = new Point(1, 1);
		trapazoidUnit[2] = new Point(-3, 0);
		trapazoidUnit[3] = new Point(1, -1);
		trapazoidUnit[4] = new Point(2, 0);
		delta = new Point(8, 2);
		pyramidColor[0] = Colour.magenta;
		pyramidColor[1] = Colour.orange;
	}

	public void fill3DRect(Canvas canvas,
			int x, int y, int width, int height,
			boolean raised) {
		int c = mPaint.getColor();
		int brighter = (new Colour(c)).brighter();
		int darker = (new Colour(c)).darker();

		Paint fillPaint = new Paint();
	    fillPaint.setColor(c);
	    fillPaint.setStyle(Style.FILL);
	    fillPaint.setAlpha(255);
		if (!raised) {
			fillPaint.setColor(darker);
		}
		Rect rect = new Rect(x + 1, y + 1, width - 2, height - 2);
		canvas.drawRect(rect, fillPaint);
		fillPaint.setColor(raised ? brighter : darker);
		canvas.drawLine(x, y, x, y + height - 1, fillPaint);
		canvas.drawLine(x + 1, y, x + width - 2, y, fillPaint);
		if (width > 2)
			canvas.drawLine(x + 1, y, x + 1, y + height - 1, fillPaint);
		if (height > 2)
			canvas.drawLine(x + 2, y + 1, x + width - 1, y + 1, fillPaint);
		fillPaint.setColor(raised ? darker : c);
		canvas.drawLine(x + 1, y + height - 1, x + width - 1, y + height - 1, fillPaint);
		canvas.drawLine(x + width - 1, y, x + width - 1, y + height - 2, fillPaint);
		if (height > 2)
			canvas.drawLine(x + 2, y + height - 2, x + width - 2, y + height - 2, fillPaint);
		if (width > 2)
			canvas.drawLine(x + width - 2, y + 1, x + width - 2, y + height - 3, fillPaint);
	}

	// From Nux http://stackoverflow.com/questions/2047573/how-to-draw-filled-polygon
	static void drawPoly(Canvas canvas, int color, Point[] points) {
	    // line at minimum...
	    if (points.length < 2) {
	        return;
	    }

	    // paint
	    Paint polyPaint = new Paint();
	    polyPaint.setColor(color);
	    polyPaint.setStyle(Style.FILL);
	    polyPaint.setAlpha(255);

	    // path
	    Path polyPath = new Path();
	    polyPath.moveTo(points[0].x, points[0].y);
	    int i, len;
	    len = points.length;
	    for (i = 0; i < len; i++) {
	        polyPath.lineTo(points[i].x, points[i].y);
	    }
	    polyPath.lineTo(points[0].x, points[0].y);

	    // draw
	    canvas.drawPath(polyPath, polyPaint);
	}

	void drawShadow(Canvas canvas,
			int startX, int startY, int sizeX, int sizeY) {
		canvas.drawLine(startX, startY, startX + sizeX - 1, startY,
			mPaint);
		canvas.drawLine(startX, startY, startX, startY + sizeY - 1,
			mPaint);
	}

	private static int SIGN(int a) {
		return ((a < 0) ? -1 : 1);
	}

	void drawPyramid(Canvas canvas, int color, int size,
			int offsetPosition) {
		Point[] trapazoidList = new Point[5];
		int k, sizey;

		k = delta.x +
			mRect.width() / 2 + offsetPosition + 1;
		trapazoidList[0] = new Point((size * SIGN(trapazoidUnit[4].x) *
				mRect.width() / (mNumber + 1)) / 2 + k,
				offsetPosition + 2);
		sizey = mRect.height() - 4;
		if (sizey <= 0) {
			trapazoidList[0].y += sizey;
			sizey = 1;
		}
		for (int l = 1; l <= 4; l++) {
			trapazoidList[l] = new Point();
			if (java.lang.Math.abs(trapazoidUnit[l].x) == 3)
				trapazoidList[l].x = (size + 1) *
				SIGN(trapazoidUnit[l].x) *
				mRect.width() / (mSize + 1) +
				trapazoidList[l - 1].x;
			else if (java.lang.Math.abs(trapazoidUnit[l].x) == 2)
				trapazoidList[l].x = size *
				SIGN(trapazoidUnit[l].x) *
				mRect.width() / (mSize + 1) +
				trapazoidList[l - 1].x;
			else
				trapazoidList[l].x = mRect.width() /
				(2 * (mSize + 1)) +
				trapazoidList[l - 1].x;
			trapazoidList[l].y = sizey *
				trapazoidUnit[l].y +
				trapazoidList[l - 1].y;
		}
		int trapazoidColor, fontColor;
		if (getMono()) {
			if ((offsetPosition != 0) ^ getReverse()) {
				trapazoidColor = tileColor;
				fontColor = new Colour(background).darker2();
			} else {
				trapazoidColor = new Colour(background).darker2();
				fontColor = tileColor;
			}
		} else {
			if ((pyramidColor[color] == new Colour(background).darker2()) &&
					offsetPosition != 0) {
				trapazoidColor = tileColor;
			} else if ((pyramidColor[color] == tileColor) &&
					offsetPosition == 0) {
				trapazoidColor = new Colour(background).darker2();
			} else {
				trapazoidColor = pyramidColor[color];
			}
			fontColor = trapazoidColor;
		}
		mPaint.setColor(trapazoidColor);
		drawPoly(canvas, trapazoidColor, trapazoidList);

		if (getMono()) {
			mPaint.setColor(fontColor);
			/*int offsetX = -1, offsetY, fontSize;
			// TODO need font for mono
			fontSize = java.lang.Math.min(geo.getTileSize().x,
				geo.getTileSize().y) / 2;
			g.setFont(new Font("arial", Font.PLAIN, fontSize));
			offsetX += -fontSize / 3;
			offsetY = fontSize / 3;
			g.drawString(Character.toString(geo.getPyramidNames().charAt(color)),
				k + offsetX,
				trapazoidListY[0] + geo.getTileSize().y / 2 + offsetY);*/
		}
	}

	@Override
	public void draw(Canvas canvas) {
		int dx, dy;
		int fillColor;
		int pressedOffset = (mPressed) ? 1 : 0;

		dx = pressedOffset;
		dy = pressedOffset;
		if (getReverse()) {
			fillColor = new Colour(background).darker();
		} else {
			fillColor = tileColor;
		}
		if (pressedOffset != 0) {
			mPaint.setColor(new Colour(fillColor).darker());
			drawShadow(canvas,
				dx - pressedOffset, dy - pressedOffset,
				mRect.width(), mRect.height());
		}
		fill3DRect(canvas, dx, dy, mRect.width(), mRect.height(),
			pressedOffset == 0);
		if (pressedOffset != 0) {
			mPaint.setColor(fillColor);
			mPaint.setStyle(Style.FILL);
			mPaint.setAlpha(255);
			Rect rectWidth = new Rect(dx + mRect.width() - 2, dy - 1,
				1, 3);
			Rect rectHeight = new Rect(dx - 1, dy + mRect.height() - 2,
				3, 1);
			canvas.drawRect(rectWidth, mPaint);
			canvas.drawRect(rectHeight, mPaint);
		}
		drawPyramid(canvas, mColor, mNumber, pressedOffset);
	}

	public void onPress() {
		if (!mPressed) {
			mPressed = !mPressed;
			invalidateSelf();
		}
	}

	public void onRelease() {
		if (mPressed) {
			mPressed = !mPressed;
			invalidateSelf();
		}
	}

	boolean getMono() {
		return mono;
	}

	boolean getReverse() {
		return reverse;
	}

	@Override
	public int getOpacity() {
		//return PixelFormat.OPAQUE;
		return PixelFormat.TRANSLUCENT;
	}

	@Override
	public void setAlpha(int arg0) {
		// TODO Auto-generated method stub
	}

	@Override
	public void setColorFilter(ColorFilter arg0) {
		// TODO Auto-generated method stub
	}
}
